#include <stdio.h>
#include "main.h"
#include "usart.h"
#include "ringbuffer.h"

RingBuffer g_ringBuffer;
#define    USART1_RB_POOL_SIZE    256
char usart1RbPool[USART1_RB_POOL_SIZE];

void USART1_Init(void)
{
    RingBufferInit(&g_ringBuffer, usart1RbPool, USART1_RB_POOL_SIZE);
}

void EnableUSART1IRQ(void)
{
    HAL_NVIC_SetPriority(USART1_IRQn, 0, 0);
	  HAL_NVIC_EnableIRQ(USART1_IRQn);
}

RingBuffer_t GetUsart1Rb(void)
{
    return &g_ringBuffer;
}


int fputc(int ch, FILE *p)
{
    static RingBuffer_t rb;
    if (!rb) {
        rb = GetUsart1Rb();
    }

    int empty = RingBufferEmpty(rb);
    RingBufferWrite(rb, (char *)&ch, sizeof(char));

    if (empty) {
        __HAL_UART_ENABLE_IT(&huart1, UART_IT_TXE);
    }
	return ch;
}

void USART1_IRQHandler(void)
{
   static RingBuffer_t rb;
   unsigned int length = 0;
   char ch = 0;
   if (!rb) {
       rb = GetUsart1Rb();
   }
   if (__HAL_UART_GET_FLAG(&huart1, UART_FLAG_TXE) != RESET) {


      RingBufferReadByte(rb, &ch, 1);
      huart1.Instance->DR = ch;

      if (RingBufferEmpty(rb)) {
          __HAL_UART_DISABLE_IT(&huart1, UART_IT_TXE);
      }
   }
    HAL_UART_IRQHandler(&huart1);
}
